{*******************************************************}
{                                                       }
{                      Tachyon Unit                     }
{    Vector Raster Geographic Information Synthesis     }
{                     VOICE  ..  Tracer                 }
{                     GRIP ICE .. Tongs                 }
{                Digital Terrain Mapping                }
{               Image Locatable Holographics            }
{                          SOS MAP                      }
{  Surreal Object Synthesis Multimedia Analysis Product }
{                   Fractal3D  Life MOW                 }
{       Copyright (c) 1995,2006  Ivan Lee Herring       }
{                                                       }
{*******************************************************}
unit gREIV;

interface

uses
  Windows, Messages, SysUtils, Classes,
  Graphics, Controls, Forms, Dialogs,
  jpeg, PNGImage, Clipbrd, Printers,
  Menus, ExtCtrls, Gauges, Buttons, ExtDlgs, ComCtrls;

type
  TREIVForm = class(TForm)
    ScrollBox1: TScrollBox;
    Image1: TImage;
    SaveDialog1: TSaveDialog;
    MainMenu1: TMainMenu;
    File1: TMenuItem;
    F_Open: TMenuItem;
    N13: TMenuItem;
    F_Save: TMenuItem;
    FS_PNG: TMenuItem;
    FS__Saveas: TMenuItem;
    FS_BMP: TMenuItem;
    FS_JPG: TMenuItem;
    N6: TMenuItem;
    Edit1: TMenuItem;
    E_Crop: TMenuItem;
    EFS_Crop256: TMenuItem;
    EFS_Crop512: TMenuItem;
    EFS_Crop1024: TMenuItem;
    E_Resize: TMenuItem;
    N8: TMenuItem;
    IZ_Rotate: TMenuItem;
    N11: TMenuItem;
    E_Copy: TMenuItem;
    E_Paste: TMenuItem;
    Panel1: TPanel;
    OpenButton: TSpeedButton;
    SaveButton: TSpeedButton;
    CropButton: TSpeedButton;
    CopyButton: TSpeedButton;
    PasteButton: TSpeedButton;
    Gauge1: TGauge;
    HelpButton: TSpeedButton;
    ExitButton: TSpeedButton;
    OpenDialog: TOpenDialog;
    LoadFileDialog: TOpenPictureDialog;
    SavePictureDialog1: TSavePictureDialog;
    StatusBar1: TStatusBar;
    Crop1: TMenuItem;
    Exit1: TMenuItem;
    Resizer: TSpeedButton;
    Rotator: TSpeedButton;
    N1: TMenuItem;
    procedure FormCreate(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure SaveButtonClick(Sender: TObject);
    procedure SaveImage;
    procedure CopyButtonClick(Sender: TObject);
    procedure PasteButtonClick(Sender: TObject);
    procedure HelpButtonClick(Sender: TObject);
    procedure EFS_Crop256Click(Sender: TObject);
    procedure EFS_Crop512Click(Sender: TObject);
    procedure EFS_Crop1024Click(Sender: TObject);
    procedure Cropola;
    procedure E_ResizeClick(Sender: TObject);
    procedure IZ_RotateClick(Sender: TObject);

    procedure Image1MouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure DrawRubberband;
    procedure Image1MouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure Image1MouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
    procedure F_OpenClick(Sender: TObject);
    procedure LoadFromFile(RFilename: string);
{    procedure ImageProgress(Sender: TObject; Stage: TProgressStage;
      PercentDone: Byte; RedrawNow: Boolean; const R: TRect;
      const Msg: string);}
    procedure Progress(Value: Byte);
    procedure F_SaveClick(Sender: TObject);
    procedure FS_PNGClick(Sender: TObject);
    procedure FS_BMPClick(Sender: TObject);
    procedure FS_JPGClick(Sender: TObject);
    procedure Exit1Click(Sender: TObject);
    procedure E_CropClick(Sender: TObject);
    procedure Crop1Click(Sender: TObject);

  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  REIVForm: TREIVForm;

implementation

{$R *.DFM}

uses gResize, gRotate, gGlobal, gGrip;

var
  Loading: Boolean = FALSE;
  LastPos: Integer;
  RFilename: string;
  {pf24bit pf32bit}
  {MyPixelFormat: TPixelFormat;
  PixelScanSize: Byte;}
(******************************************************)
(******************************************************)

function myTestImage: Boolean; {ONLY handle 24 bit colors}
var
  temp: Integer;
  dc: HDC;
begin
  dc := GetDc(0);
  if ((GetDeviceCaps(dc, BITSPIXEL) = 24) or (GetDeviceCaps(dc,
    BITSPIXEL) = 32)) then begin
    if (GetDeviceCaps(dc, BITSPIXEL) = 24) then
    begin
      MyPixelFormat := pf24bit;
      PixelScanSize := 3;
    end else begin
      DoMessages(40);
      MyPixelFormat := pf24bit;
      PixelScanSize := 3;
{    MyPixelFormat:=pf32bit;
    PixelScanSize:=4;}
    end;
    temp := ReleaseDc(0, dc); {Give back the screen dc}
    if temp <> 1 then Application.Terminate;
    myTestImage := True;
  end else
  begin {or 36}
    DoMessages(11);
    temp := ReleaseDc(0, dc);
    if temp <> 1 then Application.Terminate;
    myTestImage := False;
  end; {of if 24 bit}
end; {of this}



procedure TREIVForm.FormCreate(Sender: TObject);
begin
{If (Not myTestImage) then Application.Terminate;}
  left := REIVX;
  top := REIVY;
end;


procedure TREIVForm.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
{ If ImageLoaded then RUnLoadImage;}
  REIVX := REIVForm.left;
  REIVY := REIVForm.top;
  DoSaver;
  {FormControl:=1;
  GRIPForm.FreeForm; }
end;

procedure TREIVForm.Exit1Click(Sender: TObject);
begin
  Close;
end;

procedure TREIVForm.HelpButtonClick(Sender: TObject);
begin
  Application.HelpContext(234);
end;
(******************************************************)
(******************************************************)


(******************************************************)
(******************************************************)

procedure TREIVForm.F_OpenClick(Sender: TObject);
begin
  if OpenWithPreview then
  begin
    LoadFileDialog.InitialDir := ImagePath;
    LoadFileDialog.Filter := 'All (*.bmp;*.png)|*.bmp;*.png';
    if LoadFileDialog.Execute then
    begin
      ImagePath := ExtractFilePath(LoadFileDialog.Filename);
      RFilename := ExtractFileName(LoadFileDialog.Filename);
      LoadFromFile(RFilename);
    end;
  end else
  begin
    OpenDialog.InitialDir := ImagePath;
    OpenDialog.Filter := 'All (*.bmp;*.png)|*.bmp;*.png';
    if OpenDialog.Execute then
    begin
      ImagePath := ExtractFilePath(OpenDialog.Filename);
      RFilename := ExtractFileName(OpenDialog.Filename);
      LoadFromFile(RFilename);
    end;
  end;
end;

{Load a file}

procedure TREIVForm.LoadFromFile(RFilename: string);
var
  PNG: TPNGObject;{TPNGObject;}
begin
  LastPos := 0;
  Loading := TRUE;
  REIVForm.Cursor:=CRHOURGLASS;
  Application.ProcessMessages;
{  if ImageLoaded then RUnLoadImage;}
  FillRect(Image1.Picture.Bitmap.Canvas.Handle, ClientRect,
    COLOR_BTNFACE + 1);
  {Load the image}
  if (lowercase(extractfileext(RFilename)) = '.bmp') then begin
    Image1.Picture.LoadFromFile(ImagePath + RFilename);
  end else
  if (lowercase(extractfileext(RFilename)) = '.png') then
  begin
    PNG := TPNGObject.Create;
    {In case something goes wrong, free both PNG and Bitmap}
    try
      PNG.LoadFromFile(ImagePath + RFilename);
      Image1.Picture.Bitmap.Assign(PNG); //Convert data into bitmap
      finally
      PNG.Free;
    end;
  end;

  if (Image1.Picture.Bitmap.PixelFormat <> pf24bit)
    then
  begin
    if (Image1.Picture.Bitmap.PixelFormat = pf4bit)
      then DoMessages(41);
    if (Image1.Picture.Bitmap.PixelFormat = pf8bit)
      then DoMessages(42);
    if (Image1.Picture.Bitmap.PixelFormat = pf15bit)
      then DoMessages(43);
    if (Image1.Picture.Bitmap.PixelFormat = pf16bit)
      then DoMessages(44);
    if (Image1.Picture.Bitmap.PixelFormat = pf32bit)
      then DoMessages(45);
    Image1.Picture.Bitmap.PixelFormat := pf24bit;
  end;

  Progress(0);
  Loading := FALSE;
  Image1.Canvas.Draw(0,
    0,
    Image1.Picture.Graphic);
  with ScrollBox1.HorzScrollBar do
  begin
    Increment := 4;
    Position := 0;
    Range := Image1.Width;
    Visible := True;
  end;
  with ScrollBox1.VertScrollBar do
  begin
    Increment := 4;
    Position := 0;
    Range := Image1.Height;
    Visible := True;
  end;
{  Screen.Cursor:=CRDEFAULT;     }
  REIVForm.Cursor := CRDEFAULT;
{ StatusBar1.Panels[1].Text :='Loading Image and Stats';}
{  RmyLoadImage;
  If ImageLoaded then RSetFileName(Filename);}
end;

  (*
procedure TREIVForm.ImageProgress(Sender: TObject; Stage:
  TProgressStage;
  PercentDone: Byte; RedrawNow: Boolean; const R: TRect;
  const Msg: string);
begin
{If it is a TPNG image we can show it progressive}
  if Sender is TPNG5Image then
  begin
    Progress(PercentDone);
    if (PercentDone mod 5 = 0) and
      (TChunkIHDR(TPNG5IMAGE(SENDER).Chunks[0]).Interlaced = 0) then
    begin
      BitBlt(Image1.Picture.Bitmap.Canvas.Handle,
        (Width - TBitmap(Sender).Width) div 2,
        ((Height - TBitmap(Sender).Height) div 2) + LastPos,
        R.Right,
        R.Bottom - LastPos,
        TBitmap(Sender).Canvas.Handle, 0, LastPos, SRCCOPY);
      LastPos := R.Bottom;
    end
    else if (TChunkIHDR(TPNG5IMAGE(SENDER).Chunks[0]).Interlaced = 1)
      then
      Image1.Picture.Bitmap.Canvas.Draw((Width -
        TBitmap(Sender).Width) div 2,
        ((Height) - TBitmap(Sender).Height) div 2, TBitmap(sender));
  end;
  Application.Processmessages;
end;
*)
procedure TREIVForm.Progress(Value: Byte);
begin
  {Ensures that the value does not pass 100}
  if Value > 100 then
    Value := 100;
  {Set the panel text}
  Gauge1.Progress := Value;
  StatusBar1.Panels[1].Text := INTTOSTR(Value);
end;


(******************************************************)
(******************************************************)

procedure TREIVForm.SaveButtonClick(Sender: TObject);
begin
  SaveImage;
end;

procedure TREIVForm.SaveImage;
var
 PNG: TPNGObject;{TPNGObject;}
 tempath: string;
begin
  REIVForm.Cursor:=CRHOURGLASS;
  Application.ProcessMessages;
  tempath := ImagePath + RFilename;
  if Image1.Picture.Graphic = nil then
  begin
    DoMessages(46);
    exit;
  end;
  if (lowercase(extractfileext(RFilename)) = '.bmp') then
  begin
    Image1.Picture.Graphic.SaveToFile(tempath);
  end else
  if (lowercase(extractfileext(RFilename)) = '.png') then
  begin
    PNG := TPNGObject.Create;
    {BMP := TBitmap.Create; }
    {In case something goes wrong, free both PNG and Bitmap}
    try
      Image1.Picture.Bitmap.PixelFormat := pf24bit;
      PNG.Assign(TBitmap(Image1.Picture.Graphic)); //Convert data into png
      PNG.SaveToFile(tempath);
      finally
      PNG.Free;
    end;
  end;
  REIVForm.Cursor:=CRDefault;
  Application.ProcessMessages;
end;


procedure TREIVForm.F_SaveClick(Sender: TObject);
begin
  SaveImage;
end;

procedure TREIVForm.FS_PNGClick(Sender: TObject);
var temp, tempath: string;
{  jp: TJpegImage; }//Requires the "jpeg" unit added to "uses" clause.
begin
  tempath := ImagePath +ChangeFileExt(RFileName, '.png'); ;
  temp := ExtractFileExt(tempath);
  SaveDialog1.FileName := tempath;
  SavePictureDialog1.FileName :=  tempath;
  if ((temp = '.PNG') or (temp = '.png')) then
  begin
    if SaveWithPreview then
    begin
       {showmessage('not changed 2');}
      {set SaveDialog filter to display png's only}
      SavePictureDialog1.filter := 'PNG files|*.PNG';
      SavePictureDialog1.DefaultExt := 'PNG';
      if SavePictureDialog1.Execute then {Save it}
      begin
        temp := ExtractFileExt(SavePictureDialog1.FileName);
        if ((temp = '.PNG') or (temp = '.png')) then
        begin
          RFileName := ExtractFileName(SavePictureDialog1.FileName);
          ImagePath := ExtractFilePath(SavePictureDialog1.FileName);
          {  RSetFileName(FileName);}
          SaveImage;
        end;
      end;
    end else
    begin
        {showmessage('not changed 2');}
        {set SaveDialog filter to display png's only}
        SaveDialog1.filter := 'PNG files|*.PNG';
        SaveDialog1.DefaultExt := 'PNG';
        if SaveDialog1.Execute then {Save it}
        begin
          temp := ExtractFileExt(SaveDialog1.FileName);
          if ((temp = '.PNG') or (temp = '.png')) then begin
            RFileName := ExtractFileName(SaveDialog1.FileName);
            ImagePath := ExtractFilePath(SaveDialog1.FileName);
            {  RSetFileName(FileName);}
            SaveImage;
          end;
        end;
    end;
  end else showmessage('not changed');
  Gauge1.Progress := 0;
end;


procedure TREIVForm.FS_BMPClick(Sender: TObject);
var
  tempath: string;
  Bitmap: TBitmap;
{  SaveFilters: TEncodeFilterSet; }
begin
  if SaveWithPreview then
  begin
    tempath := ImagePath +ChangeFileExt(RFileName, '.bmp');
    SavePictureDialog1.FileName := tempath;
    SavePictureDialog1.filter := 'BMP files|*.bmp';
  {set SaveDialog Default extension}
    SavePictureDialog1.DefaultExt := 'bmp';
    if SavePictureDialog1.Execute then
    begin
      tempath := ExtractFileExt(SavePictureDialog1.FileName);
      if ((tempath = '.BMP') or (tempath = '.bmp')) then
      begin
        {Save it}
        Bitmap := TBitmap.Create;
        Bitmap.Width := Image1.Picture.Graphic.Width;
        Bitmap.Height := Image1.Picture.Graphic.Width;
        Image1.Picture.Bitmap.PixelFormat := pf24bit;
        Bitmap.Assign(Image1.Picture.Bitmap);
        Bitmap.SaveToFile(SavePictureDialog1.Filename);
        Bitmap.Free;
        {reset the gauge to 0}
        Gauge1.Progress := 0;
      end; end;
  end else
  begin
    tempath := ImagePath +ChangeFileExt(RFileName, '.bmp');
    SaveDialog1.FileName := tempath;
    SaveDialog1.FileName := ChangeFileExt(tempath, '.bmp');
    SaveDialog1.filter := 'BMP files|*.bmp';
  {set SaveDialog Default extension}
    SaveDialog1.DefaultExt := 'bmp';
    if SaveDialog1.Execute then
    begin
      tempath := ExtractFileExt(SaveDialog1.FileName);
      if ((tempath = '.BMP') or (tempath = '.bmp')) then
      begin
        {Save it}
        Bitmap := TBitmap.Create;
        Bitmap.Width := Image1.Picture.Graphic.Width;
        Bitmap.Height := Image1.Picture.Graphic.Width;
        Image1.Picture.Bitmap.PixelFormat := pf24bit;
        Bitmap.Assign(Image1.Picture.Bitmap);
        Bitmap.SaveToFile(SaveDialog1.Filename);
        Bitmap.Free;
        {reset the gauge to 0}
        Gauge1.Progress := 0;
      end; end;
  end; end;


procedure TREIVForm.FS_JPGClick(Sender: TObject);
var tempath: string;
  jp: TJpegImage; //Requires the "jpeg" unit added to "uses" clause.
begin
  if SaveWithPreview then
  begin
    tempath := ImagePath +ChangeFileExt(RFileName, '.bmp');
    SavePictureDialog1.FileName := ChangeFileExt(tempath, '.jpg');
    SavePictureDialog1.filter := 'Jpeg files|*.jpg';
    SavePictureDialog1.DefaultExt := 'jpg';
    if SavePictureDialog1.Execute then
    begin
      tempath := ExtractFileExt(SavePictureDialog1.FileName);
      if ((tempath = '.JPG') or (tempath = '.jpg')) then
      begin
        jp := TJpegImage.Create;
        try
          with jp do
          begin
            Image1.Picture.Bitmap.PixelFormat := pf24bit;
            Assign(Image1.Picture.Bitmap);
            SaveToFile(SavePictureDialog1.Filename)
          end;
        finally
          jp.Free;
        end;
      end;
  {reset the gauge to 0}
      Gauge1.Progress := 0;
    end;
  end else
  begin
    tempath := ImagePath +ChangeFileExt(RFileName, '.bmp');
    SaveDialog1.FileName := ChangeFileExt(tempath, '.jpg');
    SaveDialog1.filter := 'Jpeg files|*.jpg';
    SaveDialog1.DefaultExt := 'jpg';
    if SaveDialog1.Execute then
    begin
      tempath := ExtractFileExt(SaveDialog1.FileName);
      if ((tempath = '.JPG') or (tempath = '.jpg')) then
      begin
        jp := TJpegImage.Create;
        try
          with jp do
          begin
            Image1.Picture.Bitmap.PixelFormat := pf24bit;
            Assign(Image1.Picture.Bitmap);
            SaveToFile(SaveDialog1.Filename)
          end;
        finally
          jp.Free;
        end;
      end;
  {reset the gauge to 0}
      Gauge1.Progress := 0;
    end;
  end;
end;
(******************************************************)
(******************************************************)

{---------------------------------------------------------------------}

procedure TREIVForm.CopyButtonClick(Sender: TObject);
begin
{Copy to clipboard}
{Check to see if image is there}
  if Image1.Picture.Bitmap <> nil then
   {Copy the image to the clipboard}
    Clipboard.Assign(Image1);
   {reset the gauge to 0}
  Gauge1.Progress := 0;
end;

procedure TREIVForm.PasteButtonClick(Sender: TObject);
begin
{Paste from clipboard}
{Paste image from clipboard}
   {does the clipboard has the right format?}
  if Clipboard.HasFormat(CF_PICTURE) then
    Image1.Picture.Assign(Clipboard);
   {reset the gauge to 0}
  Gauge1.Progress := 0;
end;
{---------------------------------------------------------------------}
{------------------------------------------------------------------------}

procedure TREIVForm.IZ_RotateClick(Sender: TObject);
begin
  RotateForm := TRotateForm.Create(Application);
  If RotateForm.ShowModal = mrok then RotateForm.Free
  else RotateForm.Free;
end;
{---------------------------------------------------------------------}
{------------------------------------------------------------------------}

procedure TREIVForm.Crop1Click(Sender: TObject);
begin
{Activate a Crop Dialog that
gets the Mouse X and Y and a Size choice RG
THis enables cropping an image that will not fit on the screen
The Mouse XY is the upper left.. no box}
end;

procedure TREIVForm.E_CropClick(Sender: TObject);
begin
  StartCropping := True;
{Crop image}
{Set CropMousing Boolean True}
{Get Mouse Coordinates}
{Crop using coordinates}
{Set CropMousing Boolean False}
end;


procedure TREIVForm.EFS_Crop256Click(Sender: TObject);
begin
  if ((Image1.Picture.Bitmap.Height > 256)
    and (Image1.Picture.Bitmap.Width > 256))
    then StartCropping256 := True else DoMessages(39980);
end;

procedure TREIVForm.EFS_Crop512Click(Sender: TObject);
begin
  if ((Image1.Picture.Bitmap.Height > 512)
    and (Image1.Picture.Bitmap.Width > 512))
    then StartCropping512 := True else DoMessages(39979);
end;

procedure TREIVForm.EFS_Crop1024Click(Sender: TObject);
begin
  if ((Image1.Picture.Bitmap.Height > 1024)
    and (Image1.Picture.Bitmap.Width > 1024))
    then StartCropping1024 := True else DoMessages(39978);
end;



{---------------------------------------------------------------------}
{------------------------------------------------------------------------}

procedure TREIVForm.Image1MouseUp(Sender: TObject;
  Button: TMouseButton;
  Shift: TShiftState;
  X, Y: Integer);
var
  Temp: Integer;
begin

  if Cropping then begin
    DrawRubberband;
    StartCropping := False;
    Cropping := False;
{    ReleaseCapture;
    ReleaseDC(HWindow, DragDC);}
    {Get Sizes and change image}
    if (((pointEnd.Y) > pointStart.Y + 23)
      and ((pointEnd.X) > pointStart.X + 32)) then
      Cropola;
{    ChangeName('O','C')};
  end else
    if CropTo_256 then begin
      DrawRubberband;
      StartCropping256 := False;
      CropTo_256 := False;
      Cropola;
{    ChangeName('O','R')};
    end else
      if CropTo_512 then begin
        DrawRubberband;
        StartCropping512 := False;
        CropTo_512 := False;
        Cropola;
{    ChangeName('O','O')};
      end else
        if CropTo_1024 then begin
          DrawRubberband;
          StartCropping1024 := False;
          CropTo_1024 := False;
          Cropola;
{    ChangeName('O','P')};
        end else
          if bLeftDown then begin
            DrawRubberband;
            bLeftDown := false;
            pointEnd.X := X;
            pointEnd.Y := Y;
  {The Lower Right is the START
  Upper left is the END}
            if pointStart.X < pointEnd.X then
            begin
              Temp := pointStart.X;
              pointStart.X := pointEnd.X;
              pointEnd.X := Temp;
            end;
            if pointStart.Y < pointEnd.Y then
            begin
              Temp := pointStart.Y;
              pointStart.Y := pointEnd.Y;
              pointEnd.Y := Temp;
            end;
            Gauge1.Progress := 0;
          end;
end;
{---------------------------------------------------------------------}

procedure TREIVForm.DrawRubberband;
begin
  SetROP2(Image1.Canvas.handle, R2_NOT);
  Image1.Canvas.moveto(pointStart.x, pointStart.y);
  Image1.Canvas.lineto(pointStart.x, pointEnd.y);
  Image1.Canvas.lineto(pointEnd.x, pointEnd.y);
  Image1.Canvas.lineto(pointEnd.x, pointStart.y);
  Image1.Canvas.lineto(pointStart.x, pointStart.y);
end;

procedure TREIVForm.Image1MouseDown(Sender: TObject; Button:
  TMouseButton;
  Shift: TShiftState; X, Y: Integer);
var ImageWidthi, ImageHeighti: integer;
begin
  ImageWidthi := Image1.Width;
  ImageHeighti := Image1.Height;
  if button = mbLeft then begin
    if (StartCropping) then begin
      if (not Cropping) then begin
        Cropping := True;
        if ((X > 0) and (X < ImageWidthi)) then
          pointStart.X := X else pointStart.X := 0;
        if ((Y > 0) and (Y < ImageHeighti)) then
          pointStart.Y := Y else pointStart.Y := 0;
        pointEnd.X := pointStart.X;
        pointEnd.Y := pointStart.Y;
      end;
    end else
      if (StartCropping256) then begin
        if (not CropTo_256) then begin
          CropTo_256 := True;
          if ((X > 0) and (X < (ImageWidthi - 257))) then
            pointStart.X := X else pointStart.X := 0;
          if ((Y > 0) and (Y < (ImageHeighti - 257))) then
            pointStart.Y := Y else pointStart.Y := 0;
          pointEnd.X := (pointStart.X + 256);
          pointEnd.Y := (pointStart.Y + 256);
          DrawRubberband;
        end;
      end else
        if (StartCropping512) then begin
          if (not CropTo_512) then begin
            CropTo_512 := True;
            if ((X > 0) and (X < (ImageWidthi - 513))) then
              pointStart.X := X else pointStart.X := 0;
            if ((Y > 0) and (Y < (ImageHeighti - 513))) then
              pointStart.Y := Y else pointStart.Y := 0;
            pointEnd.X := (pointStart.X + 512);
            pointEnd.Y := (pointStart.Y + 512);
            DrawRubberband;
          end;
        end else
          if (StartCropping1024) then begin
            if (not CropTo_1024) then begin
              CropTo_1024 := True;
              if ((X > 0) and (X < (ImageWidthi - 1025))) then
                pointStart.X := X else pointStart.X := 0;
              if ((Y > 0) and (Y < (ImageHeighti - 1025))) then
                pointStart.Y := Y else pointStart.Y := 0;
              pointEnd.X := (pointStart.X + 1024);
              pointEnd.Y := (pointStart.Y + 1024);
              DrawRubberband;
            end;
          end;
  end;
end;

procedure TREIVForm.Image1MouseMove(Sender: TObject; Shift:
  TShiftState; X,
  Y: Integer);
var BlueCS, RedCS, GreenCS, XStr, YStr: string;
{  dc:hdc;}
  BlueC, RedC, GreenC: Byte;
  Colors: TColorRef;
{  Xword,Yword:Dword;}
  ImageWidthi, ImageHeighti: integer;
begin
  ImageWidthi := Image1.Width;
  ImageHeighti := Image1.Height;
{Xword:=X; Yword:=Y; }

  if Cropping then begin
    DrawRubberband;
    if ((X > pointStart.X) and (X < ImageWidthi)) then
      pointEnd.X := X else pointEnd.X := pointEnd.X;
    if ((Y > pointStart.Y) and (Y < ImageHeighti)) then
      pointEnd.Y := Y else pointEnd.Y := pointEnd.Y;
    DrawRubberband;
  end else
    if CropTo_256 then begin
      DrawRubberband;
      if ((X > 0) and (X < (ImageWidthi - 257))) then
        pointStart.X := X else pointStart.X := 0;
      if ((Y > 0) and (Y < (ImageHeighti - 257))) then
        pointStart.Y := Y else pointStart.Y := 0;
      pointEnd.X := (pointStart.X + 256);
      pointEnd.Y := (pointStart.Y + 256);
      DrawRubberband;
    end else
      if CropTo_512 then begin
        DrawRubberband;
        if ((X > 0) and (X < (ImageWidthi - 513))) then
          pointStart.X := X else pointStart.X := 0;
        if ((Y > 0) and (Y < (ImageHeighti - 513))) then
          pointStart.Y := Y else pointStart.Y := 0;
        pointEnd.X := (pointStart.X + 512);
        pointEnd.Y := (pointStart.Y + 512);
        DrawRubberband;
      end else
        if CropTo_1024 then begin
          DrawRubberband;
          if ((X > 0) and (X < (ImageWidthi - 1025))) then
            pointStart.X := X else pointStart.X := 0;
          if ((Y > 0) and (Y < (ImageHeighti - 1025))) then
            pointStart.Y := Y else pointStart.Y := 0;
          pointEnd.X := (pointStart.X + 1024);
          pointEnd.Y := (pointStart.Y + 1024);
          DrawRubberband;
        end else
          if bLeftDown then begin {MultiImage mouser}
            DrawRubberband;
            pointEnd.x := X;
            pointEnd.y := Y;
            DrawRubberband;
          end else begin {Text out the Cursor pixel colors}
            str(X, XStr); str(Y, YStr);
  {  dc := GetDc(0);  }
   { Colors:=  GetPixel(dc,X,ImageHeighti-Y); }
            Colors := Image1.Canvas.Pixels[X, Y];
            BlueC := GetBValue(Colors); Str(BlueC, BlueCS);
            RedC := GetRValue(Colors); Str(RedC, RedCS);
            GreenC := GetGValue(Colors); Str(GreenC, GreenCS);
    {dc := }{ReleaseDc(0, dc);}{Give back the screen dc}
            StatusBar1.Panels.items[1].Text :=
              'X: ' + XStr + ' Y: ' + YStr + ' B: ' + BlueCS + ' G: '
              + GreenCS + ' R: ' + RedCS;
          end;
end;
{---------------------------------------------------------------------}
{------------------------------------------------------------------------}

procedure TREIVForm.Cropola;
var
  H, W: Integer;
  Bitmap: TBitmap;
  StartPoints, EndPoints: TPoint;
begin {Create a new bitmap and set its size}
  if (pointStart.X > pointEnd.X) then
  begin W := (pointStart.X - pointEnd.X);
    StartPoints.X := pointStart.X;
    EndPoints.X := pointEnd.X;
  end else begin W := (pointEnd.X - pointStart.X);
    StartPoints.X := pointEnd.X;
    EndPoints.X := pointStart.X;
  end;
  if (pointStart.Y > pointEnd.Y) then begin
    H := (pointStart.Y - pointEnd.Y);
    StartPoints.Y := pointStart.Y;
    EndPoints.Y := pointEnd.Y;
  end else begin
    H := (pointEnd.Y - pointStart.Y);
    StartPoints.Y := pointEnd.Y;
    EndPoints.Y := pointStart.Y;
  end; ;
  Bitmap := TBitmap.Create;
  Bitmap.Width := W;
  Bitmap.Height := H;
 {Copy the palette}
  Bitmap.Palette := Image1.Picture.Bitmap.Palette;
  {Copy it}
  Bitmap.Canvas.CopyMode := cmSrcCopy;
{ CopyRect(Dest: TRect; Canvas: TCanvas; Source: TRect); }
  Bitmap.Canvas.Copyrect(Rect(0, 0, W - 1 {Bitmap.Width}, H - 1
    {Bitmap.Height}),
    Image1.Picture.Bitmap.Canvas,
{Rect(StartPoints.X, StartPoints.Y, EndPoints.X, EndPoints.Y));}
    Rect(EndPoints.X, EndPoints.Y, StartPoints.X, StartPoints.Y));

 {Copy the palette}
  Bitmap.Palette := Image1.Picture.Bitmap.Palette;
 {Assign the new smaller bitmap}
  Image1.Picture.Bitmap.Assign(Bitmap);
 {Free the bitmap}
  Bitmap.Free
end;
{---------------------------------------------------------------------}
{------------------------------------------------------------------------}

{---------------------------------------------------------------------}
{------------------------------------------------------------------------}

procedure TREIVForm.E_ResizeClick(Sender: TObject);
{Sample of how to resize an Image using delphi calls}
var
  Ratio, Ratso: Double;
  Code, NewH, NewW, H, W: Integer; HS, WS: string;
  Bitmap: TBitmap;
begin
  ResizeForm := TResizeForm.Create(Application);
  Application.Processmessages;
  {Get desired Size from dialog}
  {NewH:=144; NewW:=192;}
  { With  TResizeForm.Create(Self) do Showmodal;}
  W := Image1.Picture.Bitmap.Width;
  H := Image1.Picture.Bitmap.Height;
  Str(W, WS); Str(H, HS);
  ResizeForm.Height.Text := HS;
  ResizeForm.Width.Text := WS;
  ResizeForm.Showmodal;
  if (ResizeForm.ModalResult = mrOK) then
  begin
    HS := ResizeForm.Height.Text;
    WS := ResizeForm.Width.Text;
    val(HS, NewH, Code);
    val(WS, NewW, Code);
   {Create a new bitmap and set its size}
    W := Image1.Picture.Bitmap.Width;
    H := Image1.Picture.Bitmap.Height;
    if (W > H) then
    begin
      Ratio := (W / NewW); Ratso := (H / Ratio);
      H := trunc(Ratso);
      W := NewW;
    end else
    begin
      Ratio := (H / NewH); Ratso := (W / Ratio);
      W := trunc(Ratso);
      H := NewH;
    end;
    Bitmap := TBitmap.Create;
    Bitmap.Width := W;
    Bitmap.Height := H;
   {Copy the palette}
    Bitmap.Palette := Image1.Picture.Bitmap.Palette;
   {Delete the lines needed to shrink}
    SetStretchBltMode(Bitmap.Canvas.Handle, {HALFTONE}
      STRETCH_DELETESCANS);
   {Resize it}
    {Draws the graphic specified by the Graphic parameter
     in the rectangle specified by the Rect parameter.
     procedure StretchDraw(const Rect: TRect; Graphic: TGraphic );}
    Bitmap.Canvas.StretchDraw(Rect(0,
      0,
      W - 1 {Bitmap.Width},
      H - 1 {Bitmap.Height}),
      Image1.Picture.Bitmap);
(*
{procedure CopyRect(Dest: TRect; Canvas: TCanvas; Source: TRect);}
   Bitmap.Canvas.Copyrect(Rect(0,
                             0,
                             W-1{Bitmap.Width},
                             H-1{Bitmap.Height}),
                        Image1.Picture.Bitmap.Canvas,
                        Rect(0,
                             0,
                            Image1.Picture.Bitmap.Width,
                            Image1.Picture.Bitmap.Height));
*)
   {Copy the palette}
    Bitmap.Palette := Image1.Picture.Bitmap.Palette;
   { Assign the new smaller bitmap}
    Image1.Picture.Bitmap.Assign(Bitmap);
   {Free the bitmap}
    Bitmap.Free
  end;
  ResizeForm.Free;
end;
{---------------------------------------------------------------------}
{------------------------------------------------------------------------}


(******************************************************)
(******************************************************)





end.
